@@ -1,23 +1,19 @@
 package Zoidberg::PdParse;
 
-our $VERSION = '0.3a_pre1';
+our $VERSION = '0.40';
 
 use strict;
 
-require Exporter;
-
 use Data::Dumper;
 use IO::File;
 use Storable qw/dclone/;
+use Exporter::Tidy
+	default => [qw/pd_read pd_write pd_merge/],
+	other   => [qw/pd_read_multi pd_forget/];
 
-$Data::Dumper::Purity=1;
-$Data::Dumper::Deparse=1;
-$Data::Dumper::Indent=1;
-
-push @{Zoidberg::PdParse::ISA}, 'Exporter';
-
-@{Zoidberg::PdParse::EXPORT} = qw/pd_read pd_write pd_merge/;
-@{Zoidberg::PdParse::EXPORT_OK} = qw/pd_read_multi pd_forget/;
+$Data::Dumper::Purity = 1;
+$Data::Dumper::Deparse = 1;
+$Data::Dumper::Indent = 1;
 
 our %read = ();
 our $base_dir = './';
@@ -67,30 +63,6 @@ sub pd_write {
     $fh->print(Dumper($ref)); #print returns bit
 }
 
-sub pd_merge {
-    my $ref = shift;
-    foreach my $ding (@_) { 
-    	$ding = dclone($ding) if grep {ref($ding) eq $_} qw/HASH ARRAY SCALAR/;
-    	$ref = _merge($ref, $ding);
-    }
-    return $ref;
-}
-
-sub _merge {
-	my ($ref, $ding) = @_;
-	while (my ($k, $v) = each %{$ding}) {
-            if (defined($ref->{$k}) && ref($v)) {
-                if (ref($v) eq 'ARRAY') { # this one is open for discussion
-                        push @{$ref->{$k}}, @{$ding->{$k}};
-                }
-                elsif (grep {ref($v) eq $_} qw/SCALAR CODE Regexp/) { $ref->{$k} = $v; }
-                else { $ref->{$k} = _merge($ref->{$k}, $ding->{$k}); } #recurs for HASH (or object)
-            }
-            else { $ref->{$k} = $v; }
-        }
-	return $ref;
-}
-
 sub _file {
 	# files starting with a / are absolute
 	# else relative to base_dir
@@ -180,8 +152,8 @@ Forget read files.
 
 =head1 AUTHOR
 
-R.L. Zwart, E<lt>carlos@caremail.nlE<gt>
-Jaap Karssenberg || Pardus [Larus] E<lt>j.g.karssenberg@student.utwente.nlE<gt>
+R.L. Zwart, E<lt>rlzwart@cpan.orgE<gt>
+Jaap Karssenberg || Pardus [Larus] E<lt>pardus@cpan.orgE<gt>
 
 Copyright (c) 2002 Raoul L. Zwart. All rights reserved.
 This program is free software; you can redistribute it and/or
