@@ -14,13 +14,12 @@ $Data::Dumper::Purity=1;
 $Data::Dumper::Deparse=1;
 $Data::Dumper::Indent=1;
 
-
 push @{Makefile::ISA}, 'Exporter';
 @{Makefile::EXPORT_OK} = qw/
 	get_version get_version_from compare_version
-	path_copy file_copy
-	rmdir_force dir_copy pd_read chdir
-	pd_write
+	check_mtime path_copy file_copy
+	make_dir rmdir_force dir_copy
+	pd_read pd_write
 /;
 
 our %default_config = (
@@ -31,9 +30,14 @@ our %default_config = (
 	'manifest' => 'MANIFEST',
 	'tools' => {
 		# name => [ command, [extensions] ]
-		'perl' =>  [ $Config{perl5}, ['pl', 'PL', 'pm'] ],
+		'perl' =>  [ $Config{perl5}||$Config{perlpath}, ['pl', 'PL', 'pm'] ],
 		'shell' => [ $Config{sh},    [ 'sh' ]           ],
-	}
+	},
+	'include' => {
+		'NAME'          => 'unknown',
+		'VERSION'       => 'unknown',
+		'AUTHOR'	=> 'unknown',
+	},
 );
 
 our @var_arg = @ARGV; # copy here - use later
@@ -51,11 +55,14 @@ sub new {
 		$config->{conf_file} = $file;
 	}
 
-	$self = bless $config, $class; 
+	$self = bless $config, $class;
 	#<VUNZIG>
 	foreach (keys %default_config) { unless ($self->{$_}) { $self->{$_} = $default_config{$_}; } }
-	foreach (keys %{$default_config{vars}}) { unless ($self->{vars}{$_}) { $self->{vars}{$_} = $default_config{vars}->{$_}; } }
-	foreach (keys %{$default_config{tools}}) { unless ($self->{tools}{$_}) { $self->{tools}{$_} = $default_config{tools}->{$_}; } }
+	for my $d (qw/vars tools include/) {
+		foreach (keys %{$default_config{$d}}) { 
+			$self->{$d}{$_} = $default_config{$d}->{$_} unless $self->{$d}{$_};
+		}
+	}
 
 	my %tmp;
 	for (keys %{$self->{vars}}) { $tmp{uc($_)} = $self->{vars}{$_}; }
@@ -75,13 +82,16 @@ sub new {
 			for (@var_arg) { # parse gnu style args
 				#print "debug var: -$_-\n";
 				#$_ =~ s/^--?//;
-				if ($_ =~ m/^-{0,2}(\w*)=[\"\']?(.*)[\"\']?$/) { 
+				if ($_ =~ m/^(--usage|--help|-u|-h)$/) { $self->print_usage && exit 0 }
+				elsif ($_ =~ m/^--make_vars$/) { $self->print_vars && exit 0 }
+				elsif ($_ =~ m/^--make_targets$/) { $self->print_targets && exit 0 }
+				elsif ($_ =~ m/^-{0,2}(\w*)(?:=[\"\']?(.*)[\"\']?)?$/) {
 					if (exists $self->{$1}) { $self->{$1} = $2; }
-					else { $self->{vars}{uc($1)} = $2; }
-				}
-				elsif ($_ =~ m/^(--usage|--help|-u|-h)$/) {
-					$self->print_usage;
-					exit 0;
+					else { 
+						$self->{tools}{uc($1)}[0] = $2
+							if exists $self->{tools}{uc($1)};
+						$self->{vars}{uc($1)} = $2 || 1;
+					}
 				}
 				else {
 					print "Unknown or wrongly formatted option \"$_\"\nTry \"--usage\" for help text.\n";
@@ -116,7 +126,7 @@ sub pd_write {
 sub check_manifest {
 	my $self = shift;
 	$self->{manifest_ok} = 1;
-	print "debug: using manifest: $self->{manifest}\n";
+    # print "debug: using manifest: $self->{manifest}\n";
 	for ($self->manifest) {
 		if ($self->{vars}{VERBOSE}) { print "Checking for file \"$_\".\n"; }
 		unless (-f $_) {
@@ -125,7 +135,7 @@ sub check_manifest {
 		}
 	}
 	if ($self->{manifest_ok}) {
-		print "Manifest seems to be ok.\n";
+		print "Manifest $self->{manifest} seems to be ok.\n";
 		return 1;
 	}
 	else { 
@@ -206,13 +216,13 @@ sub write_makefile {
 					else { die "Please define a command for the \".$1\" extension" }
 				}
 				[$file.$_, $m_tool];
-			} sort grep {$_ !~ m/^(\.\.?|CVS)$/} (readdir D) ];
+			} sort grep {($_ ne 'CVS') && ($_ !~ /^\./)} (readdir D) ];
 			closedir D;
 		}
 		elsif (-x $file) { $file = "\@".$file; }
 		else { die "Shouldn't \"$file\" be executable ?"}
 		[$name, $file, $tool];
-	} grep {$_ !~ m/^(\.\.?|CVS)$/} (readdir TD);
+	} grep {($_ ne 'CVS') && ($_ !~ /^\./)} (readdir TD);
 	closedir TD;
 	push @targets, map { [$_, undef, undef] } @{$self->{fake_targets}};
 
@@ -294,11 +304,8 @@ sub serialize { # recursive
 	}
 }
 
-sub print_usage {
+sub print_header {
 	my $self = shift;
-	my $name = $self->{include}{NAME} || 'unknown';
-	my $version = $self->{include}{VERSION} || 'unknown';
-	my $author = $self->{include}{AUTHOR} || 'unknown';
 	print << "END";
 
 ## This package uses Makefile.pm - yet another approach to Makefile.PL
@@ -306,26 +313,68 @@ sub print_usage {
 
 --[ Package information ]
 
- NAME    : $name
- VERSION : $version
- AUTHOR  : $author
+ NAME    : $self->{include}{NAME}
+ VERSION : $self->{include}{VERSION}
+ AUTHOR  : $self->{include}{AUTHOR}
 
+END
+}
+
+sub print_usage {
+	my $self = shift;
+	$self->print_header;
+	print << "END";
 --[ Usage ]
 
   > perl Makefile.PL [options] [VAR1=value] [VAR2=value]
 
 --[ Options ]
 
-  --usage, --help, -u, -h Print this text.
+  --usage, -u
+  --help, -h
+        print this text.
 
---[ Variables ]
+  --make_vars
+        print all known variables
+
+  --make_targets
+        print all known targets
 
 END
+}
 
+
+sub print_vars {
+	my $self = shift;
+	
+	$self->print_header;
+	print "--[ Variables ]\n\n";
 	for (keys %{$self->{vars}}) { 
-		print '  ', $_, ($self->{vars}{$_} ? ' :  '.$self->{vars}{$_} : ''), "\n"; 
+		print '  ', $_, ($self->{vars}{$_} ? ' :  '.$self->{vars}{$_} : ''), "\n";
 	}
-	print "\n\n";
+	print "\n";
+}
+
+sub print_targets {
+	my $self = shift;
+	
+	my $config = {};
+        my $file = $self->{conf_file};
+       	if (-f $file) {
+		$config = $self->pd_read($file);
+                $config->{conf_file} = $file;
+        }
+	else {
+		print "This information gets available after writing a Makefile\nTry \"perl Makefile.PL\"\n";
+		return 1;
+	}
+
+	$self->print_header;
+	print "--[ Known make targets ]\n\n";
+	for (@{$config->{target_names}}) { 
+		print '  '.$_.($self->{help}{$_} ? ' :  '.$self->{help}{$_} : '')."\n";
+	}
+	print "\n";
 }
 
 sub compare_version {
@@ -383,8 +432,7 @@ sub get_version_from {
 		$inpod = /^=(?!cut)/ ? 1 : /^=cut/ ? 0 : $inpod;
 		next if $inpod || /^\s*#/;
 		next unless /([\$*])(([\w\:\']*)\bVERSION)\b.*\=/;
-		my $eval = qq{
-			package Makefile::_version;
+		my $eval = qq{ package Makefile::_version;
 			no strict;
 
 			local $1$2;
@@ -402,45 +450,43 @@ sub get_version_from {
 }
 
 sub path_copy {
-	my $e = 'usage: path_copy $from, $to';
-	my $to = pop @_ || die $e;
-	my $from = pop @_ || die $e;
-	unless (-e $from) { confess "no such thing \"$from\" to copy"; }
-
-	# create dir tee
-	if ($to =~ m/^(.*)\/(.*)/) {
-		my @tree = split(/\//, $1);
-		my $dir = '';
-		while (@tree) {
-			$dir .= (shift @tree).'/';
-			unless (-d $dir) { mkdir $dir }
-		}
-	}
-
+	my ($from, $to) = @_;
+	confess "no such thing \"$from\" to copy" unless -e $from;
+	if ($to =~ /(.*)\/(.*)/) { make_dir($1) }
 	return file_copy($from, $to);
 }
 
+sub make_dir {
+	my $dir = pop;
+	# create dir tee
+	my @tree = split(/\//, $dir);
+	$dir = '';
+	while (@tree) {
+		$dir .= shift(@tree).'/';
+		mkdir $dir, 0755 || die "Could not make dir $dir\n"
+			unless -d $dir;
+	}
+}
+
 sub file_copy {
-	my $e = 'usage: file_copy $from, $to;';
-	my $to = pop @_ || die $e;
-	my $from = pop @_ || die $e;
-	unless (-e $from) { die "no such thing \"$from\" to copy"; }
+	my ($from, $to) = @_;
+	confess "no such thing \"$from\" to copy" unless -e $from;
 
-	if (
-		( [stat $from]->[9] == [stat $to]->[9] )
-		&& ( -s $from == -s $to )
-	) { return 1 }
-        
 	open IN, $from || die "==> Could not open file $from\n";
-	open OUT, ">$to" || die "==> Could not open file $to\n";
-	while (<IN>) { print OUT $_; }
+	open OUT, '>', $to || die "==> Could not open file $to\n";
+	print OUT $_ for (<IN>);
 	close IN;
 	close OUT || die "==> Could not write file $to\n";
-    # set new file's mtime
-    utime(@{[stat$from]}[8,9],$to);
+
+	chmod -x $from ? 0755 : 0644, $to;
+
+	# set new file's mtime
+	utime undef, undef, $to;
 	return 1;
 }
 
+sub check_mtime { (-s $_[1] && [stat $_[0]]->[9] <= [stat $_[1]]->[9]) ? 1 : 0 }
+
 sub rmdir_force {
 	my $dir = pop || die 'usage: rmdir_force $dir;';
 	$dir =~ s/\/$//; #/
@@ -454,14 +500,11 @@ sub rmdir_force {
 }
 
 sub dir_copy {
-	# dir from, dir to
-	my $e = 'usage: dir_copy $from, $to;';
-	my $to = pop || die $e;
-	my $from = pop || die $e;
+	my ($from, $to, $make) = @_;
 
 	$from =~ s/\/?$/\//;
 	$to =~ s/\/?$/\//;
-	unless (-e $to) { mkdir($to) || die "Could not make dir $to\n"; }
+	make_dir($to) unless -d $to;
 
 	opendir FROM, $from || die "Could not open dir $from\n";
 	my @files = grep {$_ !~ m/^\.\.?$/} readdir FROM;
@@ -469,9 +512,13 @@ sub dir_copy {
 
 	my @done = @files;
 
-	foreach my $file (grep {-f $from.$_} @files) { file_copy($from.$file, $to.$file) }
+	foreach my $file (grep {-f $from.$_} @files) {
+		next if $make && check_mtime($from.$file, $to.$file);
+		file_copy($from.$file, $to.$file);
+	}
 	foreach my $dir (grep {-d $from.$_} @files) {
-		push @done, dir_copy( $from.$dir, $to.$dir ); #recurs
+		push @done, map "$dir/$_",
+			dir_copy( $from.$dir, $to.$dir ); #recurs
 	}
 
 	return @done;
@@ -541,13 +588,15 @@ can use this module to get their configuration vars.
 
 One can add make targets by simply adding scripts or directories to the target directory.
 
-For obvious reasons it is not possible to have a target called 'CVS' --
-'cvs' or any combination with it should work fine. Also it is not wise to call a target
-'Makefile' or something alike. Since Makefile.pm relies on @ARGV one should not touch
+Files and directories starting with a '.' are ignored.
+For obvious reasons targets called 'CVS'will also be ignored. Also it is not wise to call a target
+'Makefile' or something alike. 
+
+Since Makefile.pm relies on @ARGV one should not touch
 @ARGV in a target script or at least do C<require 'm/Makefile.pm';> first.
 
-Also since the module itself just glues scripts in the Makefile, it can also be used for
-all kind of packages other then perl extensions.
+Since the module itself just glues scripts in the Makefile, it can also be used for
+all kind of packages other then perl extensions. 
 
 This module is not intended for permanent installation,
 it should be included in the package, by default as F<m/Makefile.pm>.
@@ -696,6 +745,15 @@ Get version of an installed module.
 Get version declared in file. This uses the same regex as L<ExtUtils::MakeMaker> uses for
 "VERSION_FROM" this is: C</([\$*])(([\w\:\']*)\bVERSION)\b.*\=/> . See docs MM for more details.
 
+=item C<check_mtime($from, $to)> [non oo method]
+
+If this method returns true one can assume that the file $to
+doesn't need updating from $from ( think make(1) ).
+
+=item C<make_dir($dir)> [non oo method]
+
+Create all directories in the path of $dir.
+
 =item C<path_copy($from, $to)> [non oo method]
 
 Copy file $from to file $to, create directories if needed.
@@ -709,9 +767,11 @@ Copy file $from to file $to, create directories if needed.
 Like C<rm -fr dir>, used to rm non-empty dir trees.
 I<Use with care>.
 
-=item C<dir_copy($from, $to)> [non oo method]
+=item C<dir_copy($from, $to, $make)> [non oo method]
 
-Copy contents of dir $from to dir $to
+Copy contents of dir $from to dir $to. If 
+$make is set true files will only be copied if
+L<check_mtime> failes.
 
 =back
 
